﻿using Netina.Domain.Dtos.ResponseDtos.Torob;

namespace Netina.Core.CoreServices.SearchServices;

public class GetTorobProductsQueryHandler : IRequestHandler<GetTorobProductsQuery, List<TorobProductResponseDto>>
{
    private readonly IRepositoryWrapper _repositoryWrapper;
    private readonly SiteSettings _siteSetting;

    public GetTorobProductsQueryHandler(IRepositoryWrapper repositoryWrapper , IOptionsSnapshot<SiteSettings> optionsSnapshot)
    {
        _repositoryWrapper = repositoryWrapper;
        _siteSetting = optionsSnapshot.Value;
    }
    public async Task<List<TorobProductResponseDto>> Handle(GetTorobProductsQuery request, CancellationToken cancellationToken)
    {
        var products = await _repositoryWrapper.SetRepository<Product>()
            .TableNoTracking
            .OrderByDescending(p => p.ModifiedAt == DateTime.MinValue ? p.CreatedAt : p.ModifiedAt)
            .Select(ProductMapper.ProjectToTorobResponseDto)
            .Skip(request.Page * 100)
            .Take(100)
            .ToListAsync(cancellationToken);

        products.ForEach(p =>
        {
            p.page_url = $"{_siteSetting.WebSiteUrl}/products/{p.product_id}";
        });

        return products;
    }
}